@@ -25,6 +25,7 @@ import functools
 import logging
 import multiprocessing
 import os
+import shutil
 import subprocess
 import sys
 import tempfile
@@ -38,6 +39,7 @@ import xcffib.xproto
 
 import libqtile.config
 from libqtile import command, ipc
+from libqtile.backend import base
 from libqtile.backend.x11.core import Core
 from libqtile.confreader import Config
 from libqtile.core.manager import Qtile
@@ -117,15 +119,6 @@ def can_connect_qtile(socket_path, *, ok=None):
     return False
 
 
-def whereis(program):
-    """Search PATH for executable"""
-    for path in os.environ.get('PATH', '').split(':'):
-        if os.path.exists(os.path.join(path, program)) and \
-           not os.path.isdir(os.path.join(path, program)):
-            return os.path.join(path, program)
-    return None
-
-
 class BareConfig(Config):
     auto_fullscreen = True
     groups = [
@@ -193,7 +186,7 @@ class Xephyr:
 
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, _exc_type, _exc_val, _exc_tb):
         self.stop_xephyr()
 
     def start_xephyr(self):
@@ -437,15 +430,15 @@ class TestManager:
         return self.test_window(name, "notification")
 
     def test_xclock(self):
-        path = whereis("xclock")
+        path = shutil.which("xclock")
         return self._spawn_window(path)
 
     def test_xeyes(self):
-        path = whereis("xeyes")
+        path = shutil.which("xeyes")
         return self._spawn_window(path)
 
     def test_xcalc(self):
-        path = whereis("xcalc")
+        path = shutil.which("xcalc")
         return self._spawn_window(path)
 
     def groupconsistency(self):
@@ -475,12 +468,12 @@ def xvfb():
 
 
 @pytest.fixture(scope="session")
-def display(xvfb):
+def display(xvfb):  # noqa: F841
     return os.environ["DISPLAY"]
 
 
 @pytest.fixture(scope="session")
-def xephyr(request, xvfb):
+def xephyr(request, xvfb):  # noqa: F841
     kwargs = getattr(request, "param", {})
 
     with Xephyr(**kwargs) as x:
@@ -518,3 +511,19 @@ def manager_nospawn(request, xephyr):
 
 
 no_xinerama = pytest.mark.parametrize("xephyr", [{"xinerama": False}], indirect=True)
+
+
+@pytest.fixture(scope="function")
+def fake_window():
+    """
+    A fake window that can provide a fake drawer to test widgets.
+    """
+    class FakeWindow:
+        class _NestedWindow:
+            wid = 10
+        window = _NestedWindow()
+
+        def create_drawer(self, width, height):
+            return base.Drawer(None, self, width, height)
+
+    return FakeWindow()
