@@ -38,8 +38,7 @@ import libqtile.confreader
 import libqtile.hook
 import libqtile.layout
 import libqtile.widget
-import libqtile.window
-from libqtile.backend.x11 import xcbq
+from libqtile.backend.x11 import window, xcbq
 from libqtile.command.client import SelectError
 from libqtile.command.interface import CommandError, CommandException
 from libqtile.config import Match
@@ -47,6 +46,7 @@ from libqtile.confreader import Config
 from libqtile.lazy import lazy
 from test import conftest
 from test.conftest import BareConfig, Retry, no_xinerama
+from test.layouts.layout_utils import assert_focused
 
 
 class ManagerConfig(Config):
@@ -85,6 +85,7 @@ class ManagerConfig(Config):
     screens = [libqtile.config.Screen(
         bottom=libqtile.bar.Bar(
             [
+                libqtile.widget.Prompt(),
                 libqtile.widget.GroupBox(),
             ],
             20
@@ -259,7 +260,17 @@ class _ChordsConfig(Config):
                 "j",
                 lazy.layout.down(),
             )
-        ], "test")
+        ], "test"),
+        libqtile.config.KeyChord(["control"], "d", [
+            libqtile.config.KeyChord([], "a", [
+                libqtile.config.KeyChord([], "1", [
+                    libqtile.config.Key([], "u", lazy.ungrab_chord()),
+                    libqtile.config.Key([], "v", lazy.ungrab_all_chords()),
+                    libqtile.config.Key([], "j", lazy.layout.down()),
+                ], "inner_named"),
+            ]),
+            libqtile.config.Key([], "z", lazy.layout.down()),
+        ], "nesting_test"),
     ]
     mouse = []
     screens = [libqtile.config.Screen(
@@ -286,7 +297,7 @@ def test_immediate_chord(manager):
     # use normal bind to shift focus up
     manager.c.simulate_keypress([], "k")
     assert manager.c.groups()["a"]["focus"] == "two"
-    # enter into key chord and "k" bindin no longer working
+    # enter into key chord and "k" binding no longer working
     manager.c.simulate_keypress(["control"], "a")
     manager.c.simulate_keypress([], "k")
     assert manager.c.groups()["a"]["focus"] == "two"
@@ -314,7 +325,7 @@ def test_mode_chord(manager):
     # use normal bind to shift focus up
     manager.c.simulate_keypress([], "k")
     assert manager.c.groups()["a"]["focus"] == "two"
-    # enter into key chord and "k" bindin no longer working
+    # enter into key chord and "k" binding no longer working
     manager.c.simulate_keypress(["control"], "b")
     manager.c.simulate_keypress([], "k")
     assert manager.c.groups()["a"]["focus"] == "two"
@@ -336,6 +347,50 @@ def test_mode_chord(manager):
     assert manager.c.groups()["a"]["focus"] == "one"
 
 
+@chords_config
+@no_xinerama
+def test_chord_stack(manager):
+    manager.test_window("two")
+    manager.test_window("one")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    manager.c.simulate_keypress(["control"], "d")  # ["nesting_test"]
+    # "z" should work, "k" shouldn't:
+    manager.c.simulate_keypress([], "z")
+    assert manager.c.groups()["a"]["focus"] == "two"
+    manager.c.simulate_keypress([], "z")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    manager.c.simulate_keypress([], "k")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    # enter ["nesting_test", "", "inner_named"]:
+    manager.c.simulate_keypress([], "a")
+    manager.c.simulate_keypress([], "1")
+    # "j" should work:
+    manager.c.simulate_keypress([], "j")
+    assert manager.c.groups()["a"]["focus"] == "two"
+    manager.c.simulate_keypress([], "j")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    # leave "inner_named" ~> ["nesting_test"]:
+    manager.c.simulate_keypress([], "u")
+    manager.c.simulate_keypress([], "z")
+    assert manager.c.groups()["a"]["focus"] == "two"
+    manager.c.simulate_keypress([], "z")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    manager.c.simulate_keypress([], "k")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    # enter ["nesting_test", "", "inner_named"]:
+    manager.c.simulate_keypress([], "a")
+    manager.c.simulate_keypress([], "1")
+    # leave all: ~> []
+    manager.c.simulate_keypress([], "v")
+    # "k" should work, "z" shouldn't:
+    manager.c.simulate_keypress([], "k")
+    assert manager.c.groups()["a"]["focus"] == "two"
+    manager.c.simulate_keypress([], "k")
+    assert manager.c.groups()["a"]["focus"] == "one"
+    manager.c.simulate_keypress([], "z")
+    assert manager.c.groups()["a"]["focus"] == "one"
+
+
 @manager_config
 @no_xinerama
 def test_spawn(manager):
@@ -412,7 +467,7 @@ def test_change_state_via_message(manager):
     window_info = manager.c.window.info()
     conn = xcbq.Connection(manager.display)
 
-    data = xcffib.xproto.ClientMessageData.synthetic([libqtile.window.IconicState, 0, 0, 0, 0], "IIIII")
+    data = xcffib.xproto.ClientMessageData.synthetic([window.IconicState, 0, 0, 0, 0], "IIIII")
     ev = xcffib.xproto.ClientMessageEvent.synthetic(
         32, window_info["id"], conn.atoms['WM_CHANGE_STATE'], data
     )
@@ -420,7 +475,7 @@ def test_change_state_via_message(manager):
     conn.xsync()
     assert manager.c.window.info()["minimized"]
 
-    data = xcffib.xproto.ClientMessageData.synthetic([libqtile.window.NormalState, 0, 0, 0, 0], "IIIII")
+    data = xcffib.xproto.ClientMessageData.synthetic([window.NormalState, 0, 0, 0, 0], "IIIII")
     ev = xcffib.xproto.ClientMessageEvent.synthetic(
         32, window_info["id"], conn.atoms['WM_CHANGE_STATE'], data
     )
@@ -623,6 +678,40 @@ def test_default_float(manager):
         w.kill_client()
         conn.finalize()
 
+    w = None
+    conn = xcbq.Connection(manager.display)
+
+    def size_hints():
+        nonlocal w
+        w = conn.create_window(5, 5, 10, 10)
+
+        # set the aspect hints
+        hints = [0] * 18
+        hints[0] = xcbq.NormalHintsFlags["PAspect"]
+        hints[11] = hints[12] = hints[13] = hints[14] = 1
+        w.set_property("WM_NORMAL_HINTS", hints, type="WM_SIZE_HINTS", format=32)
+        w.map()
+        conn.conn.flush()
+
+    try:
+        manager.create_window(size_hints)
+        assert manager.c.window.info()['floating'] is True
+        info = manager.c.window.info()
+        assert info['width'] == 10
+        assert info['height'] == 10
+        manager.c.window.toggle_floating()
+        assert manager.c.window.info()['floating'] is False
+        info = manager.c.window.info()
+        assert info['width'] == 398
+        assert info['height'] == 578
+        manager.c.window.toggle_fullscreen()
+        info = manager.c.window.info()
+        assert info['width'] == 800
+        assert info['height'] == 600
+    finally:
+        w.kill_client()
+        conn.finalize()
+
 
 @manager_config
 @no_xinerama
@@ -1114,46 +1203,24 @@ class _Config(Config):
     auto_fullscreen = True
 
 
-class ClientNewStaticConfig(_Config):
-    @staticmethod
-    def main(c):
-        def client_new(c):
-            c.cmd_static(0)
-        libqtile.hook.subscribe.client_new(client_new)
-
-
-clientnew_config = pytest.mark.parametrize("manager", [ClientNewStaticConfig], indirect=True)
-
-
-@clientnew_config
-def test_clientnew_config(manager):
-    a = manager.test_window("one")
-    manager.kill_window(a)
-
-
-class ToGroupConfig(_Config):
-    @staticmethod
-    def main(c):
-        def client_new(c):
-            c.togroup("d")
-        libqtile.hook.subscribe.client_new(client_new)
-
-
-togroup_config = pytest.mark.parametrize("manager", [ToGroupConfig], indirect=True)
-
-
-@togroup_config
-def test_togroup_config(manager):
-    manager.c.group["d"].toscreen()
+@manager_config
+def test_labelgroup(manager):
     manager.c.group["a"].toscreen()
-    a = manager.test_window("one")
-    assert len(manager.c.group["d"].info()["windows"]) == 1
-    manager.kill_window(a)
+    assert manager.c.group["a"].info()["label"] == "a"
+
+    manager.c.labelgroup()
+    manager.c.widget["prompt"].fake_keypress("b")
+    manager.c.widget["prompt"].fake_keypress("Return")
+    assert manager.c.group["a"].info()["label"] == "b"
+
+    manager.c.labelgroup()
+    manager.c.widget["prompt"].fake_keypress("Return")
+    assert manager.c.group["a"].info()["label"] == "a"
 
 
 @manager_config
 def test_color_pixel(manager):
-    (success, e) = manager.c.eval("self.conn.color_pixel(\"ffffff\")")
+    (success, e) = manager.c.eval("self.core.conn.color_pixel(\"ffffff\")")
     assert success, e
 
 
@@ -1414,17 +1481,33 @@ def test_hints_setting_unsetting(manager):
 
 @manager_config
 def test_strut_handling(manager):
-    w = None
+    w = []
     conn = xcbq.Connection(manager.display)
 
     def has_struts():
         nonlocal w
-        w = conn.create_window(0, 0, 10, 10)
-        w.set_property("_NET_WM_STRUT", [0, 0, 0, 10])
-        w.map()
+        w.append(conn.create_window(0, 0, 10, 10))
+        w[-1].set_property("_NET_WM_STRUT_PARTIAL", [0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 800])
+        w[-1].map()
+        conn.conn.flush()
+
+    def with_gaps_left():
+        nonlocal w
+        w.append(conn.create_window(800, 0, 10, 10))
+        w[-1].set_property("_NET_WM_STRUT_PARTIAL", [820, 0, 0, 0, 0, 480, 0, 0, 0, 0, 0, 0])
+        w[-1].map()
+        conn.conn.flush()
+
+    def with_gaps_bottom():
+        nonlocal w
+        w.append(conn.create_window(800, 0, 10, 10))
+        w[-1].set_property("_NET_WM_STRUT_PARTIAL", [0, 0, 0, 130, 0, 0, 0, 0, 0, 0, 800, 1440])
+        w[-1].map()
         conn.conn.flush()
 
     def test_initial_state():
+        while manager.c.screen.info()["index"] != 0:
+            manager.c.next_screen()
         assert manager.c.window.info()['width'] == 798
         assert manager.c.window.info()['height'] == 578
         assert manager.c.window.info()['x'] == 0
@@ -1433,13 +1516,22 @@ def test_strut_handling(manager):
         bar = manager.c.window[bar_id].info()
         assert bar["height"] == 20
         assert bar["y"] == 580
+        manager.c.next_screen()
+        assert manager.c.window.info()['width'] == 638
+        assert manager.c.window.info()['height'] == 478
+        assert manager.c.window.info()['x'] == 800
+        assert manager.c.window.info()['y'] == 0
 
+    manager.test_xcalc()
+    manager.c.next_screen()
     manager.test_xcalc()
     test_initial_state()
 
     try:
+        while manager.c.screen.info()["index"] != 0:
+            manager.c.next_screen()
         manager.create_window(has_struts)
-        manager.c.window.static(0, None, None, None, None)
+        manager.c.window.static(None, None, None, None, None)
         assert manager.c.window.info()['width'] == 798
         assert manager.c.window.info()['height'] == 568
         assert manager.c.window.info()['x'] == 0
@@ -1449,8 +1541,18 @@ def test_strut_handling(manager):
         assert bar["height"] == 20
         assert bar["y"] == 570
 
+        manager.c.next_screen()
+        manager.create_window(with_gaps_bottom)
+        manager.c.window.static(None, None, None, None, None)
+        manager.create_window(with_gaps_left)
+        manager.c.window.static(None, None, None, None, None)
+        assert manager.c.window.info()['width'] == 618
+        assert manager.c.window.info()['height'] == 468
+        assert manager.c.window.info()['x'] == 820
+        assert manager.c.window.info()['y'] == 0
     finally:
-        w.kill_client()
+        for win in w:
+            win.kill_client()
         conn.finalize()
 
     test_initial_state()
@@ -1592,3 +1694,45 @@ def test_cursor_warp(manager):
     assert p.win_x == 25
     assert p.win_y == 25
     assert p.same_screen
+
+
+def test_switch_groups_cursor_warp(manager_nospawn):
+    config = ManagerConfig
+    config.cursor_warp = True
+    config.layouts = [libqtile.layout.Stack(num_stacks=2), libqtile.layout.Max()]
+    config.groups = [libqtile.config.Group("a"), libqtile.config.Group("b", layout="max")]
+
+    manager_nospawn.start(config)
+
+    manager_nospawn.test_window("one")
+    manager_nospawn.test_window("two")
+    manager_nospawn.c.layout.previous()
+
+    assert_focused(manager_nospawn, "one")
+    assert manager_nospawn.c.group.info()["name"] == "a"
+    assert manager_nospawn.c.layout.info()["name"] == "stack"
+
+    manager_nospawn.c.group["b"].toscreen()
+
+    manager_nospawn.test_window("three")
+
+    assert_focused(manager_nospawn, "three")
+    assert manager_nospawn.c.group.info()["name"] == "b"
+    assert manager_nospawn.c.layout.info()["name"] == "max"
+
+    # do a fast switch to trigger races in focus behavior; unfortunately we
+    # need the window in layout 'b' to map quite slowly (e.g. like firefox or
+    # something), which it does not here most of the time.
+    manager_nospawn.c.group["a"].toscreen()
+    manager_nospawn.c.group["b"].toscreen()
+    manager_nospawn.c.group["a"].toscreen()
+
+    # make sure the right things are still focused
+    assert_focused(manager_nospawn, "one")
+    assert manager_nospawn.c.group.info()["name"] == "a"
+    assert manager_nospawn.c.layout.info()["name"] == "stack"
+
+    manager_nospawn.c.group["b"].toscreen()
+    assert_focused(manager_nospawn, "three")
+    assert manager_nospawn.c.group.info()["name"] == "b"
+    assert manager_nospawn.c.layout.info()["name"] == "max"
